/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

/* ============================================== */
#include <string.h>
#include <errno.h>
#include <net/ethernet.h>
#include <linux/if_packet.h>
#include <sys/socket.h>

#include "lwip/etharp.h"

/* ============================================== */

#include "lwip/opt.h"

#include "lwip/netif.h"
#include "lwip/ip_addr.h"
#include "lwip/tcpip.h"
#include "examples/example_app/default_netif.h"

static struct netif netif;
void low_level_input(struct netif *netif);

/* =========================================================== */
int socket_raw;
unsigned char mac[6] = {0x08, 0x00, 0x27, 0x1c, 0xdb, 0x5c}; /* TODO - change conform your MAC*/
int interface_index = 2;  /* TODO change conform your index*/
/* =========================================================== */

static err_t low_level_output(struct netif *netif_state, struct pbuf *p)
{
  char buf[1514];
  /* ssize_t written; */

  LWIP_ASSERT("check", &netif == netif_state);
  /* initiate transfer(); */
  pbuf_copy_partial(p, buf, p->tot_len, 0);

  if (send(socket_raw, buf, p->tot_len, 0) > 0 ) {
    return ERR_OK;
  } else {
    printf("send - %s. Socket=%d\n", strerror(errno), socket_raw);
    return ERR_IF;
  }
}

static err_t netif_tst_setup (struct netif *netif) {

  struct sockaddr_ll linkLevelInfo;

  socket_raw = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
  if (socket_raw < 0) {
      printf("Open socket error=%s\n\n", strerror(errno));
      exit(-1);
  }

  /* bind to eth:    */
  memset(&linkLevelInfo, 0, sizeof(linkLevelInfo));
  linkLevelInfo.sll_family = AF_PACKET;
  linkLevelInfo.sll_ifindex =  interface_index;
  linkLevelInfo.sll_protocol = htons(ETH_P_ALL);

  linkLevelInfo.sll_halen = ETH_ALEN;
  memcpy (linkLevelInfo.sll_addr, mac, ETH_ALEN);

  if (bind(socket_raw, (struct sockaddr*)(&linkLevelInfo), sizeof(linkLevelInfo)) != 0){
    printf("Bind socket error\n\n");
    exit(-1);
  }

  netif->name[0] = 'e';
  netif->name[1] = '0';
  netif->output = etharp_output;
  netif->linkoutput = low_level_output;
  netif->mtu = 1500;
  netif->hwaddr_len = 6;
  netif->flags = NETIF_FLAG_BROADCAST | NETIF_FLAG_ETHARP | NETIF_FLAG_IGMP | NETIF_FLAG_LINK_UP;
  memcpy (netif->hwaddr, mac, sizeof(mac));

  return ERR_OK;
}
static unsigned int counter_rx = 0;

void low_level_input(struct netif *netif)
{
  struct pbuf *p;
  u16_t len;
  char buf[4*1514];
  struct eth_hdr *ethhdr;


  /* Obtain the size of the packet and put it into the "len"
     variable. */
  len = recv(socket_raw, buf, sizeof(buf), 0);
  if (len == (u16_t)-1) {
    perror("read returned -1");
    exit(1);
  }
  printf("Recved - %d (len=%d)\n", ++counter_rx, len);

  /* We allocate a pbuf chain of pbufs from the pool. */
  p = pbuf_alloc(PBUF_RAW, len, PBUF_POOL);
  if (p != NULL) {
    pbuf_take(p, buf, len);
    /* acknowledge that packet has been read(); */

    ethhdr = (struct eth_hdr *)p->payload;
    switch(lwip_htons(ethhdr->type)) {
    case ETHTYPE_IP:
    case ETHTYPE_ARP:
        if (netif->input(p, netif) != ERR_OK) {
            pbuf_free(p);
            printf("Pacote descartado\n");
        }
        break;
    default:
        pbuf_free(p);
        printf("Tipo invalido de eth-header: %d. Pacote descartado", lwip_htons(ethhdr->type));
        break;
    }
  } else {
    /* drop packet(); */
    printf("input: could not allocate pbuf\n");
  }
}

#if LWIP_IPV4
#define NETIF_ADDRS ipaddr, netmask, gw,
void init_default_netif(const ip4_addr_t *ipaddr, const ip4_addr_t *netmask, const ip4_addr_t *gw)
#else
#define NETIF_ADDRS
void init_default_netif(void)
#endif
{

  netif_add(&netif, ipaddr, netmask, gw, &netif, netif_tst_setup, tcpip_input);
  netif_set_default(&netif);
  netif_set_up(&netif);

}

void
default_netif_poll(void)
{
  low_level_input(&netif);
}

void
default_netif_shutdown(void)
{
}
